
# q-expansion of newform 98.10.c.i, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 98
weight = 10
poly_data = [1849, 0, 43, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 8, 0, 0], 1], [[0, 0, 1, 0], 43], [[0, 0, 0, 8], 43]]

hecke_ring_character_values = [[3, [0, 0, 1, 0]]]
aps_data = [[0, 0, -16, 0], [0, 1, 0, 0], [0, -35, 0, -35], [0, 0, 0, 0], [-9380, 0, -9380, 0], [0, 0, 0, 3413], [0, 1866, 0, 0], [0, -10727, 0, -10727], [0, 0, -978936, 0], [-4317214, 0, 0, 0], [0, 152014, 0, 0], [0, 0, 2714394, 0], [0, 0, 0, 171606], [-34755692, 0, 0, 0], [0, -804002, 0, -804002], [-68067926, 0, -68067926, 0], [0, -652537, 0, 0], [0, 3161153, 0, 3161153], [-242944420, 0, -242944420, 0], [-94292464, 0, 0, 0], [0, 2541664, 0, 0], [0, 0, 677625160, 0], [0, 0, 0, 8412459], [0, -10584880, 0, -10584880], [0, 0, 0, -24189050], [0, 26082959, 0, 0], [0, 3662854, 0, 3662854], [0, 0, -1048549940, 0], [-2037576178, 0, -2037576178, 0], [-470998718, 0, 0, 0], [2875981904, 0, 0, 0], [0, -113822379, 0, -113822379], [1397099338, 0, 1397099338, 0], [0, 0, 0, 151846989], [0, 0, -11026047754, 0], [3944737688, 0, 3944737688, 0], [0, 128926457, 0, 0], [0, 0, -2944734132, 0], [0, 0, 0, -303212450], [0, -196911109, 0, -196911109], [-10802531748, 0, -10802531748, 0], [0, 0, 0, 117031657], [0, 0, -9387113016, 0], [-9064087634, 0, -9064087634, 0], [-7311024186, 0, 0, 0], [0, -28793642, 0, 0], [-18914721004, 0, 0, 0], [0, 0, 0, -1024384628], [0, -124159541, 0, 0], [0, 397548115, 0, 397548115], [0, 0, 19155851990, 0], [-25864759344, 0, 0, 0], [0, 613436206, 0, 0], [0, 0, 0, -461841355], [0, -679649200, 0, -679649200], [119368973920, 0, 119368973920, 0], [0, -2012604343, 0, 0], [0, -1201932848, 0, -1201932848], [88295272490, 0, 88295272490, 0], [-14221587162, 0, 0, 0], [0, 2748619759, 0, 0], [0, 0, 0, -1409018901], [0, 0, 0, 2129938627], [0, -4213521272, 0, 0], [0, 2051612802, 0, 2051612802], [0, 0, -325634388354, 0], [0, 0, -169544526268, 0], [-126756308770, 0, 0, 0], [179107727580, 0, 179107727580, 0], [0, 0, 0, -3314480043], [0, -1722277188, 0, 0], [0, 0, -183949685240, 0], [0, -6887452436, 0, 0], [0, 0, 162420555766, 0], [-10783189356, 0, 0, 0], [0, -13302770010, 0, -13302770010], [598959409158, 0, 598959409158, 0], [0, 7017970615, 0, 7017970615], [0, 0, 111219062814, 0], [0, -8433766194, 0, 0], [0, 0, 0, 14392000673], [431285177670, 0, 0, 0], [-1013751491296, 0, -1013751491296, 0], [0, 0, 0, 11494019270], [0, -8659730112, 0, -8659730112], [0, 0, 1008887291724, 0], [15107012610, 0, 0, 0], [0, 0, -1033797235078, 0], [0, 0, 0, -11745787359], [887424857624, 0, 0, 0], [0, 33597003365, 0, 33597003365], [0, -2386853514, 0, 0], [-1919069226472, 0, -1919069226472, 0], [-1298013748596, 0, 0, 0], [0, 0, -426075730620, 0], [0, 0, 0, -33489029756], [0, -49377931489, 0, -49377931489], [0, -18369010166, 0, 0], [0, -11970943295, 0, -11970943295], [0, 0, -3115729658882, 0], [1737655288972, 0, 0, 0], [4384264938162, 0, 4384264938162, 0], [0, 49662845161, 0, 0], [0, 0, -1020516137706, 0], [-302031918372, 0, -302031918372, 0], [0, -13793166668, 0, 0], [0, 0, 0, 80329026953], [0, 107183492796, 0, 107183492796], [-765733610784, 0, -765733610784, 0], [0, 0, 0, -18369026632], [0, -43556364984, 0, -43556364984], [-1194594009722, 0, -1194594009722, 0], [-3405744729626, 0, 0, 0], [0, -308104229, 0, 0], [-4727350470768, 0, 0, 0], [1834696742930, 0, 1834696742930, 0], [0, 0, 0, 57343764849], [0, -97285787754, 0, 0], [0, 0, 4123416186002, 0], [-7146118477172, 0, 0, 0], [0, 143656930871, 0, 0], [2712808583966, 0, 0, 0], [0, 32054777031, 0, 32054777031], [-3579368635564, 0, -3579368635564, 0], [0, 80320265717, 0, 80320265717], [-6323823163934, 0, 0, 0], [0, 0, -8013204686054, 0], [0, -35226733350, 0, -35226733350], [0, 0, 0, 182042824010], [0, 46755777317, 0, 46755777317], [-10331390498092, 0, -10331390498092, 0], [2900977466712, 0, 0, 0], [0, 0, 1755322278672, 0], [12883493919466, 0, 0, 0], [0, 37581117790, 0, 37581117790], [0, 0, 0, 160957020770], [0, 243854705943, 0, 0], [0, 279898168939, 0, 0], [0, 0, 0, -258265084527], [14323047880614, 0, 14323047880614, 0], [0, 0, 0, -399709403221], [0, 0, -14533780835338, 0], [-18353198394784, 0, -18353198394784, 0], [-778350465012, 0, 0, 0], [0, -186718889441, 0, 0], [0, 0, 0, 45558557466], [0, 0, 0, 259923850373], [0, 62330447790, 0, 0], [0, -3742615569, 0, -3742615569], [0, 0, -9033510097464, 0], [0, 0, 7053130540818, 0], [0, 0, 0, 642205455344], [23689132148884, 0, 0, 0], [0, 417455907510, 0, 417455907510], [-14179712007196, 0, -14179712007196, 0], [-5445669812528, 0, 0, 0], [0, 0, 27558337495248, 0], [0, 193995542550, 0, 193995542550], [0, 0, 0, -375206004168], [0, -144117520521, 0, 0], [0, 0, 24237014007884, 0], [26941124348154, 0, 0, 0], [21351312434008, 0, 0, 0], [0, -11840588639, 0, -11840588639], [21287974209106, 0, 21287974209106, 0], [0, 107832513506, 0, 0], [18245753854168, 0, 18245753854168, 0], [0, -184392679657, 0, 0]]
